/*
** CREAVIEW.SQL
**
**  This scripts creates all the VIEWS for the Credit database.
**  Don't forget to say 'USE Credit' first.
**
**  One of these views is deliberately inefficient.  Rewording it
**  is one of the lab exercises.  Do not attempt to fix it here.
*/

PRINT 'Begin CREAVIEW.SQL'
GO

USE Credit
SET NOCOUNT ON
GO
IF ( (SELECT db_name()) <> 'Credit')
BEGIN
   RAISERROR ('Database "Credit" not found.',22,127) WITH LOG
END
GO


/*    
** If the objects already exist (i.e. if this is a rebuild), drop them.
*/

IF EXISTS ( SELECT name FROM sysobjects
            WHERE type = 'V' AND name = 'corp_member')
    DROP VIEW corp_member

IF EXISTS ( SELECT name FROM sysobjects
            WHERE type = 'V' AND name = 'basic_member')
    DROP VIEW basic_member

IF EXISTS ( SELECT name FROM sysobjects
            WHERE type = 'V' AND name = 'statement_wide')
    DROP VIEW statement_wide

IF EXISTS ( SELECT name FROM sysobjects
            WHERE type = 'V' AND name = 'payment_wide')
    DROP VIEW payment_wide

IF EXISTS ( SELECT name FROM sysobjects
            WHERE type = 'V' AND name = 'charge_wide')
    DROP VIEW charge_wide

IF EXISTS ( SELECT name FROM sysobjects
            WHERE type = 'V' AND name = 'overdue')
    DROP VIEW overdue
GO



/*
**  Create views for the Credit database;
**    basic_member  -  Members, who do NOT have corporate cards.
**    corp_member  -  Members, who have their cards through a corporation.
**    statement_wide  -  Statements, widened to include member info.
**    payment_wide  -  Payments, widened to include member info.
**    charge_wide  -  charges, widened to include member and provider info.
**    overdue  -  Statement_wide, overdue.
**
**  One of these views is deliberately inefficient.  Rewording it
**  is one of the lab exercises.  Do not attempt to fix it here.
*/



CREATE VIEW corp_member
AS
    SELECT 
         member.member_no
      ,  member.lastname
      ,  member.firstname
      ,  member.middleinitial
      ,  corporation.corp_no          
      ,  corporation.corp_name             
      ,  corporation.street           
      ,  corporation.city             
      ,  corporation.state_prov            
      ,  corporation.mail_code              
      ,  corporation.phone_no         
      ,  corporation.expr_dt        
      ,  corporation.region_no        
      ,  corporation.corp_code
    FROM member, corporation
    WHERE corporation.corp_no = member.corp_no
GO



CREATE VIEW basic_member
AS
    SELECT 
         member.member_no
      ,  member.lastname
      ,  member.firstname
      ,  member.middleinitial
      ,  member.street
      ,  member.city
      ,  member.state_prov
      ,  member.mail_code
      ,  member.phone_no
      ,  member.region_no
      ,  member.expr_dt
      ,  member.member_code
    FROM member
    WHERE member_no NOT IN (SELECT member_no FROM corp_member)
GO



CREATE VIEW statement_wide
AS
    SELECT 
         member.member_no
      ,  member.lastname
      ,  member.firstname
      ,  member.middleinitial
      ,  member.street
      ,  member.city
      ,  member.state_prov
      ,  member.mail_code
      ,  member.phone_no
      ,  member.expr_dt
      ,  member.member_code
      ,  region.region_no
      ,  region.region_name
      ,  statement.statement_no     
      ,  statement.statement_dt     
      ,  statement.due_dt           
      ,  statement.statement_amt    
      ,  statement.statement_code      
    FROM member, region, statement
    WHERE member.member_no = statement.member_no 
      AND region.region_no = member.region_no
GO



CREATE VIEW payment_wide
AS
    SELECT 
         member.member_no
      ,  member.lastname
      ,  member.firstname
      ,  member.middleinitial
      ,  member.street
      ,  member.city
      ,  member.state_prov
      ,  member.mail_code
      ,  member.phone_no
      ,  member.expr_dt
      ,  member.member_code
      ,  region.region_no
      ,  region.region_name
      ,  payment.payment_no       
      ,  payment.payment_dt       
      ,  payment.payment_amt      
      ,  payment.payment_code
    FROM member, region, payment
    WHERE member.member_no = payment.member_no 
      AND region.region_no = member.region_no
GO



CREATE VIEW charge_wide
AS
    SELECT 
         member.member_no
      ,  member.lastname
      ,  member.firstname
      ,  region.region_no
      ,  region.region_name
      ,  provider.provider_name
      ,  category.category_desc
      ,  charge.charge_no        
      ,  charge.provider_no      
      ,  charge.category_no      
      ,  charge.charge_dt        
      ,  charge.charge_amt       
      ,  charge.charge_code
    FROM provider, member, region, category, charge
    WHERE member.member_no = charge.member_no 
      AND region.region_no = member.region_no
      AND provider.provider_no = charge.provider_no
      AND category.category_no = charge.category_no
GO



CREATE VIEW overdue
AS
    SELECT *
    FROM statement_wide
    WHERE due_dt < GETDATE()
GO



/*
** Display results.
*/

/*
SELECT name
FROM sysobjects
WHERE type = 'V'
ORDER BY type, name
*/
GO
